package cn.com.ctop.kuaishou.modules.ai.controller;

import cn.com.ctop.common.module.service.IUserAllocationService;
import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.kuaishou.modules.ai.entity.AiKuaishouAdvertiserStrategy;
import cn.com.ctop.kuaishou.modules.ai.entity.KuaishouProjectStrategy;
import cn.com.ctop.kuaishou.modules.ai.service.IAiKuaishouAdvertiserStrategyService;
import cn.com.ctop.kuaishou.modules.ai.service.IAiKuaishouProjectCreateCreativeService;
import cn.com.ctop.kuaishou.modules.ai.service.IKuaishouProjectStrategyService;
import com.alibaba.fastjson.JSONObject;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.text.ParseException;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

@Slf4j
@Api(tags = "快手-自动投放")
@RestController
@RequestMapping("/ai/projectCreate")
public class AiKuaishouProjectCreateCreativeController {
    @Autowired
    private IAiKuaishouProjectCreateCreativeService projectCreateCreativeService;
    @Autowired
    private IAiKuaishouAdvertiserStrategyService strategyService;
    @Autowired
    private IKuaishouProjectStrategyService kuaishouProjectStrategyService;
    @Autowired
    private IUserAllocationService userAllocationService;


    static ExecutorService executorService = Executors.newFixedThreadPool(2);
    static ExecutorService kuaishouCustomCreativeSupplementExecutorService = Executors.newFixedThreadPool(2);
    static ExecutorService kuaishouProgramCreativeAutoService = Executors.newFixedThreadPool(2);
    static ExecutorService kuaishouProgramTopCreativeAutoService = Executors.newFixedThreadPool(2);

    /**
     * 项目自动创建
     *
     * @param id
     * @return
     */
    @GetMapping(value = "/projectAutomaticCreates")
    public Result<Object> projectAutomaticCreates(String id) {
        try {
            if (Check.isNull(id)) {
                throw new Exception("主键id不能为空");
            }
            KuaishouProjectStrategy strategy = kuaishouProjectStrategyService.getById(id);
            if (Check.isNull(strategy)) {
                throw new Exception("根据id获取详细信息为空");
            }
            Long projectId = strategy.getProjectId();
            List<Long> list = userAllocationService.queryAutomaticAccounts(projectId);
            if (Check.isNull(list)) {
                return Result.ok("项目：" + projectId + "中未配置自动投放账户！");
            }
            list.forEach(accountId -> executorService.submit(() -> {
                Boolean unitOverrun = projectCreateCreativeService.getUnitOverrun(projectId.toString().concat(accountId.toString()));
                if (!unitOverrun) {
                    strategy.setAccountId(accountId);
                    projectCreateCreativeService.projectAutomaticCreates(strategy);
                }
            }));
            return Result.ok("异步创建中...");
        } catch (Exception e) {
            log.error("项目自动创建异常", e.fillInStackTrace());
            return Result.error(e.getMessage());
        }
    }


    /**
     * 自动上新
     *
     * @param id
     * @return
     */
    @GetMapping(value = "/customCreativeLimit")
    public JSONObject customCreativeLimit(String id) {
        JSONObject returnJson = new JSONObject();
        try {
            if (Check.isNull(id)) {
                throw new Exception("主键id不能为空");
            }
            KuaishouProjectStrategy strategy = kuaishouProjectStrategyService.getById(id);
            if (Check.isNull(strategy)) {
                throw new Exception("根据id获取详细信息为空");
            }
            Boolean unitOverrun = projectCreateCreativeService.getUnitOverrun(strategy.getAccountId().toString());
            if (unitOverrun) {
                log.error("组创建超限,accountId:{}", strategy.getAccountId());
                throw new Exception("今日组创建已超限");
            }

            executorService.submit(() ->
                    projectCreateCreativeService.customCreativeLimit(strategy)
            );
            returnJson.put("code", 0);
            returnJson.put("message", "异步创建中");
        } catch (Exception e) {
            returnJson.put("code", -1);
            returnJson.put("message", e.getMessage());
        }
        return returnJson;

    }

    /**
     * 自定义创意补充
     * @param id
     * @param hour
     * @return
     */
    @GetMapping(value = "/kuaishouCustomCreativeSupplement")
    public JSONObject kuaishouCustomCreativeSupplement(String id, Integer hour) {
        JSONObject returnJson = new JSONObject();
        try {
            if (Check.isNull(id)) {
                throw new Exception("主键id不能为空");
            }
            KuaishouProjectStrategy strategy = kuaishouProjectStrategyService.getById(id);
            if (Check.isNull(strategy)) {
                throw new Exception("根据id获取详细信息为空");
            }
            Boolean unitOverrun = projectCreateCreativeService.getUnitOverrun(strategy.getAccountId().toString());
            if (unitOverrun) {
                log.error("组创建超限,accountId:{}", strategy.getAccountId());
                throw new Exception("今日组创建已超限");
            }
            if (Check.isNull(hour)) {
                throw new Exception("小时数据为空");
            }
            kuaishouCustomCreativeSupplementExecutorService.submit(() -> {
                projectCreateCreativeService.customCreativeSupplement(strategy, hour);
            });
            returnJson.put("code", 0);
            returnJson.put("message", "异步创建中");
        } catch (Exception e) {
            returnJson.put("code", -1);
            returnJson.put("message", e.getMessage());
        }
        return returnJson;

    }

}
